/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 
package de.bsvrz.vew.engvew.verwaltung;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dambach.dav.daf.EmpfaengerMitAnmeldung;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.engvew.engstelle.Baustelle;
import de.bsvrz.vew.engvew.engstelle.Engstelle;
import de.bsvrz.vew.engvew.engstelle.Situation;
import de.bsvrz.vew.engvew.engstelle.Unfall;

/**
 * Klasse zur Verwaltung von Engstellen in VerkehrsModellNetzen.<br>
 * Beim Instanziieren eines Klassenobjekts werden aus einem Objekt vom Typ 
 * typ.netz rekursiv die Engstellen aus allen enthaltenen VerkehrsModellNetzen
 * zur Verwaltung eingelesen. 
 * Die Mengen der Engstellen (Baustellen, Unfll) werden fr alle VerkehrsModellNetze 
 * berwacht, desweiteren wird die Invalidierung (Lschung) von Engstellen berwacht.<p>
 * Diese Klasse stellt ausserdem die aktuellen Parameterwerte der 
 * atg.engstellenVerwaltungParameter zur Verfgung.  
 * 
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:14:30 $ / ($Author: Sans $)
 */
public class VerwaltungEngstellen extends VerwaltungSituationen 
  implements EmpfaengerMitAnmeldung
{  
  
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;
  
  /**
   * Konfigurationsobjekt zur Engstellenverwaltung
   */
  private ConfigurationObject _konfigObjekt;
  
  /**
   * Nimmt den Wert des Parameters MaxVerkehrsStrkeGesperrteFahrStreifen auf 
   */
  private int _maxVerkehrsStaerkeGesperrteFahrStreifen = -1;
  
  /** 
   * Nimmt den Wert des Parameters FaktorMaxVerkehrsStrkeStromAbwrts auf 
   */
  private float _faktorVerkehrsStaerkeStromAbwaerts = -1;
  
  /**
   * Nimmt den Wert des Parameters MaxLngeStauberBaustellenEnde auf 
   */
  private long _maxLaengeStauUeberBaustellenEnde = -1;
  
  /**
   * Nimmt den Wert des Parameters WarteZeitMeldungNachRumungUnfall auf 
   */
  private long _warteZeitMeldungNachRaeumungUnfall = - 1;
  
  /**
   * Stauverwaltung
   */
  private VerwaltungStaus _verwaltungStaus;
  
  /**
   * Erzeuge eine Instanz vom Typ <code>VerwaltungEngstellen</code>
   * 
   * @param connection die Datenverteiler-Verbindung
   * @param netz das (Wurzel-) Objekt vom Typ typ.netz
   * @param konfigObjekt das Konfigurationsobjekt vom Typ typ.engstellenVerwaltung
   */
  public VerwaltungEngstellen(ClientDavInterface connection, 
      ConfigurationObject netz, ConfigurationObject konfigObjekt)
  {
    _connection = connection;
    _konfigObjekt = konfigObjekt;
    
    anmelden();
    
    _verwaltungStaus = new VerwaltungStaus(connection, netz);
    
    if (!initialisiereNetz(netz))
    {
      throw new IllegalArgumentException("Das bergebene Netz " + netz.getName() +
      " enthlt KEINE VerkehrsModellNetze");
    }
  }
  
  /**
   * Bestimmt das relevante Stau-Verwaltungsobjekt
   *  
   * @return das relevante Stau-Verwaltungsobjekt
   */
  public VerwaltungStaus getVerwaltungsStaus()
  {
    return _verwaltungStaus;
  }

  @Override
  protected void initialisiereVerkehrsModellNetz(ConfigurationObject vmn)
  {
    int errCount = 0;
    
    try
    {
      initialisiereMenge(vmn, "Baustellen");  
    }
    catch(RuntimeException e)
    {
      _debug.error("Fehler beim Initialisieren der Menge \"Baustellen\" von " + vmn, e);
      errCount+=1;
    }
    
    try
    {
      initialisiereMenge(vmn, "Unflle");
    }
    catch(RuntimeException e)
    {
      _debug.error("Fehler beim Initialisieren der Menge \"Unflle\" von " + vmn, e);
      errCount+=1;
    }
    
    if (!(errCount < 2))
    {
      throw new RuntimeException("Es konnte keine Engstellen-Menge initialisiert werden");
    }
  }
  
  @Override
  protected Engstelle erzeugeSituation(SystemObject objekt)
  {
    Engstelle engstelle = null; 
    
    if (objekt.isOfType("typ.baustelle"))
    {
      engstelle = new Baustelle(this, objekt);
      _debug.fine("Baustelle erzeugt fr " + objekt);
    }
    else if (objekt.isOfType("typ.unfall"))
    {  
      engstelle = new Unfall(this, objekt);
      _debug.fine("Unfall erzeugt fr " + objekt);
    }
    else
    {
      _debug.warning("Objekt mit PID " + objekt.getPid() + 
      " ist vom nicht untersttztem Typ " + objekt.getType());
    }
    
    if (engstelle != null)
    {
      engstelle.anmelden();
    }
    
    return engstelle;
  }  
  
  @Override
  protected void bereinigeSituation(Situation situation)
  {
    if (situation instanceof Engstelle)
    {
      _debug.fine("Bereinige Engstelle fr " + situation.getObjekt());
      ((Engstelle)situation).abmelden();
      
      _verwaltungStaus.getBereinigungsDaten().remove(((Engstelle)situation));
    }
  }
  
  /**
   * Bestimmt den Parameter MaxVerkehrsStrkeGesperrteFahrStreifen der 
   * ATG EngstellenVerwaltungParameter
   * 
   *  @return den Wert des Parameters
   */
  public int getMaxVerkehrsStaerkeGesperrteFahrStreifen()
  {
    return _maxVerkehrsStaerkeGesperrteFahrStreifen;
  }
  
  /**
   * Bestimmt den Parameter FaktorMaxVerkehrsStrkeStromAbwrts der 
   * ATG EngstellenVerwaltungParameter
   * 
   *  @return den Wert des Parameters
   */
  public float getFaktorVerkehrsStaerkeStromAbwaerts()
  {
    return _faktorVerkehrsStaerkeStromAbwaerts;
  }
    
  /**
   * Bestimmt den Parameter MaxLngeStauberBaustellenEnde der 
   * ATG EngstellenVerwaltungParameter
   * 
   *  @return den Wert des Parameters
   */
  public long getMaxLaengeStauUeberBaustellenEnde()
  {
    return _maxLaengeStauUeberBaustellenEnde;
  }
  
  /**
   * Bestimmt den Parameter WarteZeitMeldungNachRumungUnfall der 
   * ATG EngstellenVerwaltungParameter
   * 
   *  @return den Wert des Parameters
   */
  public long getWarteZeitMeldungNachRaeumungUnfall()
  {
    return _warteZeitMeldungNachRaeumungUnfall;
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.vew.vewallg.empf.EmpfaengerMitAnmeldung#getConnection()
   */
  public ClientDavInterface getConnection()
  {
    return _connection;
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.vew.vewallg.empf.EmpfaengerMitAnmeldung#getObjekt()
   */
  public ConfigurationObject getObjekt()
  {
    return _konfigObjekt;
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.vew.vewallg.empf.EmpfaengerMitAnmeldung#anmelden()
   */
  public void anmelden()
  {
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.engstellenVerwaltungParameter");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
    DataDescription datenbeschreibung = new DataDescription(atg, asp);
    
    getConnection().subscribeReceiver(this, getObjekt(), datenbeschreibung, 
        ReceiveOptions.normal(), ReceiverRole.receiver());
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
   */
  public void update(ResultData[] resultData)
  {
    // Ausgaben
    for (ResultData data : resultData)
    {
      Aspect asp = data.getDataDescription().getAspect();
      AttributeGroup atg = data.getDataDescription().getAttributeGroup();
      short sv = data.getDataDescription().getSimulationVariant();
      
      _debug.finest("Daten empfangen mit " + atg.getPid() + "/" +
          asp.getPid() + "/" + sv  + ":\n" + data);
      
      if (asp.getPid().equals("asp.parameterSoll") && 
          atg.getPid().equals("atg.engstellenVerwaltungParameter"))
      {
        if (data.hasData())
        {
          _debug.finer("Bearbeite Daten mit " + atg.getPid() + "/" +
              asp.getPid() + "/" + sv + ":\n" + data);
          
          Data item = data.getData().getItem("MaxVerkehrsStrkeGesperrteFahrStreifen");
          _maxVerkehrsStaerkeGesperrteFahrStreifen = item.asUnscaledValue().intValue(); 
          
          item = data.getData().getItem("FaktorMaxVerkehrsStrkeStromabwrts");
          _faktorVerkehrsStaerkeStromAbwaerts = item.asScaledValue().floatValue();
          
          item = data.getData().getItem("MaxLngeStauberBaustellenEnde");
          _maxLaengeStauUeberBaustellenEnde = item.asUnscaledValue().longValue();
          
          item = data.getData().getItem("WartezeitMeldungNachRumungUnfall");
          _warteZeitMeldungNachRaeumungUnfall = item.asTimeValue().getMillis();
        }
        else
        {
          _maxVerkehrsStaerkeGesperrteFahrStreifen = -1;
          _faktorVerkehrsStaerkeStromAbwaerts = -1;
          _maxLaengeStauUeberBaustellenEnde = -1;
          _warteZeitMeldungNachRaeumungUnfall = -1;
        }
        _debug.fine("Update MaxVerkehrsStrkeGesperrteFahrStreifen fr " + 
            getObjekt().getName() + " Wert ist " + _maxVerkehrsStaerkeGesperrteFahrStreifen);
        _debug.fine("Update FaktorMaxVerkehrsStrkeStromabwrts fr " + 
            getObjekt().getName() + " Wert ist " + _faktorVerkehrsStaerkeStromAbwaerts);
        _debug.fine("Update MaxLngeStauberBaustellenEnde fr " + 
            getObjekt().getName() + " Wert ist " + _maxLaengeStauUeberBaustellenEnde);
        _debug.fine("Update WartezeitMeldungNachRumungUnfall fr " + 
            getObjekt().getName() + " Wert ist " + _warteZeitMeldungNachRaeumungUnfall);
      }
    }
  }
  
}
